/*
 * Copyright (C) Freescale Semiconductor, Inc. 2006-2007
 *
 * Author: Scott Wood <scottwood@freescale.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#if defined(CONFIG_OF_LIBFDT)
#include <libfdt.h>
#endif
#include <pci.h>
#include <mpc83xx.h>
#include <vsc7385.h>
#include <ns16550.h>
#include <nand.h>

DECLARE_GLOBAL_DATA_PTR;

int board_early_init_f(void)
{
#ifndef CFG_8313ERDB_BROKEN_PMC
	volatile immap_t *im = (immap_t *)CFG_IMMR;

	if (im->pmc.pmccr1 & PMCCR1_POWER_OFF)
		gd->flags |= GD_FLG_SILENT;
#endif

	return 0;
}

#ifndef CONFIG_NAND_SPL
int checkboard(void)
{
	return 0;
}

static struct pci_region pci_regions[] = {
	{
		bus_start: CFG_PCI1_MEM_BASE,
		phys_start: CFG_PCI1_MEM_PHYS,
		size: CFG_PCI1_MEM_SIZE,
		flags: PCI_REGION_MEM | PCI_REGION_PREFETCH
	},
	{
		bus_start: CFG_PCI1_MMIO_BASE,
		phys_start: CFG_PCI1_MMIO_PHYS,
		size: CFG_PCI1_MMIO_SIZE,
		flags: PCI_REGION_MEM
	},
	{
		bus_start: CFG_PCI1_IO_BASE,
		phys_start: CFG_PCI1_IO_PHYS,
		size: CFG_PCI1_IO_SIZE,
		flags: PCI_REGION_IO
	}
};

extern int fpga_prog_fpga( char * );
void pci_init_board(void)
{
	volatile immap_t *immr = (volatile immap_t *)CFG_IMMR;
	volatile clk83xx_t *clk = (volatile clk83xx_t *)&immr->clk;
	volatile law83xx_t *pci_law = immr->sysconf.pcilaw;
	struct pci_region *reg[] = { pci_regions };
	int warmboot;
	size_t cnt;
	int r;

#if 0
	/* Erase nand. Enable this when FPGA XSVF file is corrupt and prevents board from booting */
	nand_erase_options_t opts;

	memset(&opts, 0, sizeof(opts));
	opts.offset = 0;
	opts.length = 0x100000;
	opts.jffs2  = 0;
	opts.quiet  = 0;

	nand_erase_opts(&nand_info[0], &opts);
#endif

	/* Enable all 3 PCI_CLK_OUTPUTs. */
	clk->occr |= 0xe0000000;

	/* Setup FPGA */
	cnt = 0x40000;
	r = nand_read(&nand_info[0], 0xA0000, &cnt, (u_char *)0x400000);
	if (r != 0) {
		puts("** Read error for FPGA\n");
		return;
	}
	fpga_prog_fpga((char *)0x400000);

	/*
	 * Configure PCI Local Access Windows
	 */
	pci_law[0].bar = CFG_PCI1_MEM_PHYS & LAWBAR_BAR;
	pci_law[0].ar = LBLAWAR_EN | LBLAWAR_512MB;

	pci_law[1].bar = CFG_PCI1_IO_PHYS & LAWBAR_BAR;
	pci_law[1].ar = LBLAWAR_EN | LBLAWAR_1MB;

	warmboot = gd->bd->bi_bootflags & BOOTFLAG_WARM;
#ifndef CFG_8313ERDB_BROKEN_PMC
	warmboot |= immr->pmc.pmccr1 & PMCCR1_POWER_OFF;
#endif

#ifdef CONFIG_PCI
	mpc83xx_pci_init(1, reg, warmboot);
#endif
}

#if defined(CONFIG_OF_BOARD_SETUP)
void ft_board_setup(void *blob, bd_t *bd)
{
	ft_cpu_setup(blob, bd);
#ifdef CONFIG_PCI
	ft_pci_setup(blob, bd);
#endif
}
#endif

#define FPGA_ADDR_SPACE    0x90000000
void bootm_led_failed(void)
{
	u32 reg;
	volatile u32* led_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x4C);

	/* Blink for 10 seconds */
	reg = *led_reg;
	while(1) {
		/* FPGA is little endian!!! */
		reg ^= 0x80000000;
		*led_reg = reg;
		udelay(175000);
	}
}

int check_switch(void)
{
	u32 reg;
	volatile u32* status_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x58);
	volatile u32* led_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x4C);
	int i;

	reg = *status_reg;
	if ((reg & 0x00000002) != 0) {
		/* Blink for 10 seconds */
		reg = *led_reg;
		for (i=0; i<20; i++) {
			/* FPGA is little endian!!! */
			reg ^= 0x40000000;
			*led_reg = reg;
			udelay(500000);
		}
		return 1;
	}
	return 0;
	
}

void toggle_update_led(void)
{
	u32 reg;
	volatile u32* led_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x4C);

	reg = *led_reg;
	if ((reg & 0x40000000) != 0) {
		reg &= ~0x40000000;
		reg |= 0x80000000;
	} else {
		reg &= ~0x80000000;
		reg |= 0x40000000;
	}

	*led_reg = reg;
}

void boot_next_led(void)
{
	volatile u32* led_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x4C);
	u32 reg;

	reg = *led_reg;

	if ((reg & 0x40000000) != 0) {
		reg &= ~0x40000000;
		reg |= 0x00010000;
	} else if ((reg & 0x00010000) != 0) {
		reg &= ~0x00010000;
		reg |= 0x00020000;
	} else if ((reg & 0x00020000) != 0) {
		reg &= ~0x00020000;
		reg |= 0x80000000;
	} else {
		reg &= ~0x80000000;
		reg |= 0x40000000;
	}
	*led_reg = reg;
}

void update_fail_led(void)
{
	u32 reg;
	volatile u32* led_reg = (u32 *)((void *)FPGA_ADDR_SPACE + 0x4C);

	/* Switch on all 4 leds */
	reg = *led_reg;
	reg |= 0xC0030000;
	*led_reg = reg;
}

#else /* CONFIG_NAND_SPL */
int checkboard(void) {
	return 0;
}

void board_init_f(ulong bootflag)
{
	board_early_init_f();
	NS16550_init((NS16550_t)(CFG_IMMR + 0x4600),
	             CFG_NS16550_CLK / 16 / CONFIG_BAUDRATE);
	puts("NAND boot... ");
	init_timebase();
	initdram(0);
	relocate_code(CFG_NAND_U_BOOT_RELOC + 0x10000, (gd_t *)gd,
	              CFG_NAND_U_BOOT_RELOC);
}

void board_init_r(gd_t *gd, ulong dest_addr)
{
	nand_boot();
}

void putc(char c)
{
	if (gd->flags & GD_FLG_SILENT)
		return;

	if (c == '\n')
		NS16550_putc((NS16550_t)(CFG_IMMR + 0x4600), '\r');

	NS16550_putc((NS16550_t)(CFG_IMMR + 0x4600), c);
}

#if defined(CONFIG_WATCHDOG)
void watchdog_reset (void)
{
	/* Reset the 83xx watchdog */
	volatile immap_t *immr = (immap_t *) CFG_IMMR;
	immr->wdt.swsrr = 0x556c;
	immr->wdt.swsrr = 0xaa39;
}
#endif
#endif
